#!/usr/bin/python
docstring='''seq2HTE.py A.seq B.seq HTE.txt
    find the closest homolog pair in all 4 HTE datasets (feature_[1-4].txt)
    output mapping result to HTE.txt
'''

import sys, os
import subprocess

HTE_dict=dict(
    feature_1="Hu et al",
    feature_2="Rajagopala et al",
    feature_3="Arifuzzaman et al",
    feature_4="Butland et al",
)

rootdir=os.path.dirname(os.path.abspath(__file__))

def seq2bnumber(seq_file):
    ''' map input fasta file to bnumber.fasta. 
    bnumber are ranked in ascending order of localID.
    only bnumber(s) with reciprocal seqID>=0.9 will be returned. '''
    bnumber_list=[]
    cmd="%s/NWalign %s %s/bnumber.fasta 10|sort -k5n"%(rootdir,seq_file,rootdir)
    p=subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE)
    stdout,stderr=p.communicate()
    for line in stdout.splitlines():
        query,bnumber,seqID1,seqID2,localID=line.split('\t')
        seqID1 =float( seqID1)
        seqID2 =float( seqID2)
        localID=float(localID)
        if float(seqID1)<0.9 or float(seqID2)<0.9:
            continue
        bnumber_list.append((bnumber,seqID1,seqID2,localID))
    return bnumber_list

def bnumber2hte(bnumberA_list,bnumberB_list,featureID):
    ''' map bnumberA_list and bnumberB_list to HTE dataset
    return the top bnumber hit and the PPI status: 
        NA - not in dataset
        N  - non-PPI pair
        Y  - PPI pair
    '''
    bnumberA='NA'
    bnumberB='NA'
    score=0
    PPI='NA'

    #### load positive set ####
    fullset_list=[]
    positivepair_list=[]
    filename="%s/%s.positive"%(rootdir,featureID)
    sys.stdout.write("loading %s\n"%filename)
    fp=open(filename,'r')
    for line in fp.read().splitlines():
        hit1,hit2=line.strip().split()
        fullset_list.append(hit1)
        fullset_list.append(hit2)
        positivepair_list.append(hit1+'\t'+hit2)
        positivepair_list.append(hit2+'\t'+hit1)
    fp.close()
    positivepair_set=set(positivepair_list)

    #### load full set ####
    filename="%s/%s.full"%(rootdir,featureID)
    sys.stdout.write("loading %s\n"%filename)
    fp=open(filename,'r')
    for line in fp.read().splitlines():
        fullset_list.append(line.strip())
    fp.close()
    fullset_set=set(fullset_list)

    #### perform pair mapping ####
    sys.stdout.write("mapping pair\n")
    for bA,s1A,s2A,slA in bnumberA_list:
        if not bA in fullset_set:
            continue
        for bB,s1B,s2B,slB in bnumberB_list:
            if not bB in fullset_set:
                continue
            if slA*slB<score:
                continue
            if bA+'\t'+bB in positivepair_set:
                score=slA*slB
                bnumberA="%s(%.1f%%)"%(bA,100.*slA)
                bnumberB="%s(%.1f%%)"%(bB,100.*slB)
                PPI='Y'
            elif slA*slB>score:
                score=slA*slB
                bnumberA="%s(%.1f%%)"%(bA,100.*slA)
                bnumberB="%s(%.1f%%)"%(bB,100.*slB)
                PPI='N'

    #### perform single bnumber mapping ####
    if bnumberA=="NA":
        for bA,s1A,s2A,slA in bnumberA_list:
            if bA in fullset_set:
                bnumberA="%s(%.1f)"%(bA,100.*slA)
    if bnumberB=='NA':
        for bB,s1B,s2B,slB in bnumberB_list:
            if bB in fullset_set:
                bnumberB="%s(%.1f)"%(bB,100.*slB)
    return bnumberA,bnumberB,PPI

if __name__=="__main__":
    if len(sys.argv)!=4:
        sys.stderr.write(docstring)
        exit(0)

    seqA_file=os.path.abspath(sys.argv[1])
    seqB_file=os.path.abspath(sys.argv[2])
    out_file =os.path.abspath(sys.argv[3])

    sys.stdout.write("mapping %s\n"%seqA_file)
    bnumberA_list=seq2bnumber(seqA_file)
    sys.stdout.write("mapping %s\n"%seqB_file)
    bnumberB_list=seq2bnumber(seqB_file)

    txt="#\tbnumber(seqID)\tbnumber(seqID)\tPPI\tDataset\n"
    for i in range(1,5):
        featureID="feature_%d"%i
        bnumberA,bnumberB,PPI=bnumber2hte(bnumberA_list,bnumberB_list,featureID)
        txt+="%d\t%s\t%s\t%s\t%s\n"%(i,bnumberA,bnumberB,PPI,HTE_dict[featureID])
    fp=open(out_file,'w')
    fp.write(txt)
    fp.close()
